"""
PROGRAMIRANJE OGRANICENJA

Programiranje ogranicenja je vid deklarativne paradigme koja nad 
skupom promenljivih, trazi dopustiva resenja. Pod dopustivim resenjem se 
smatraju one vrednosti promenljivih koje zadovoljavaju zadata ogranicenja.
U ovoj paradigmi zadaju se promenljive, njihovi domeni (moguce vrednosti) i 
ogranicenja koje opisuju jedan problem. Nije zadat algoritam resavanja koje
treba ispratiti, vec se propusta resavacu da pronadje odgovarajuca resenja.

Ogranicenja nisu kao u imperativnoj paradigmi, gde njihova vrednost moze biti 
>, <, ==, vec to moze biti i odnos izmedju promenljivih.


U Python-u se mora ukljuciti constraint biblioteka na jedan od dva nacina:
import constraint -> ovako ukljucena biblioteka zahteva da se pri koriscenju funkcija iz
                     biblioteke pristupa preko constraint.imeFje()

ili
from constraint import * -> ovako ukljucena biblioteka ne zahteva koriscenje imena pre tacke

ukoliko na sistemu ne postoji constraint biblioteka potrebno je instalirati
pip install python-constraint
"""


"""
1. Zadatak
Resiti kombinatorni problem:
  TWO
 +TWO
-----
 FOUR
gde je TWO trocifreni broj i FOUR cetvorocifreni broj
"""

import constraint 

problem = constraint.Problem()

problem.addVariables("TF", range(1, 10))
problem.addVariables("WOUR", range(0, 10))

def ogranicenje(t, w, o, f, u, r):
    if 2*(t*100 + w*10 + o) == f*1000 + o*100 + u*10 + r:
        return True

problem.addConstraint(ogranicenje, "TWOFUR")
resenja = problem.getSolutions()

for resenje in resenja:
    print("\n------")
    print("  {0:d}{1:d}{2:d}".format(resenje['T'], resenje['W'], resenje['O']))
    print(" +{0:d}{1:d}{2:d}".format(resenje['T'], resenje['W'], resenje['O']))
    print("------")
    print(" {0:d}{1:d}{2:d}{3:d}".format(resenje['F'], resenje['O'], resenje['U'], resenje['R']))

"""
Definisemo problem
problem = constraint .Problem ()

Dodajemo promenljive

problem.addVariable(ime_promenljive , domen_lista)
problem.addVariables(lista_imena_promenljivih , domen_lista)

# Ispisujemo resenja
print(problem.getSolutions ())

Dodajemo ogranicenja

problem.addConstraint(ogranicenje [, redosled_promenljivih ])

ogranicenje moze biti:
constraint.AllDifferentConstraint () - razlicite vrednosti svih promenljivih
constraint.AllEqualConstraint () - iste vrednosti svih promenljivih
constraint.MaxSumConstraint(s [,tezine ]) - suma vrednosti promenljivih (pomnozena
sa tezinama) ne prelazi s
constraint.MinSumConstraint(s [,tezine ]) - suma vrednosti promenljivih (pomnozena
sa tezinama) nije manja od s
constraint.ExactSumConstraint(s [,tezine ]) - suma vrednosti promenljivih (
pomnozena sa tezinama) je s
constraint.InSetConstraint(skup) - vrednosti promenljivih se nalaze u skupu skup
constraint.NotInSetConstraint(skup) - vrednosti promenljivih se ne nalaze u skupu
skup
constraint.SomeInSetConstraint(skup) - vrednosti nekih promenljivih se nalaze u
skupu skup
constraint.SomeNotInSetConstraint(skup) - vrednosti nekih promenljivih se ne
nalaze u skupu skup

redosled_promenljivih predstavlja listu promenljivih
i zadaje se zbog definisanja tacnog redosleda
ogranicenja koja se primenjuju na promenljive

Mozemo napraviti i svoju funkciju ogranicenja
Prosledjujemo funkciju ogranicenja i redosled promenljivih koji treba da odgovara
redosledu argumenata funkcije ogranicenja

"""